#!/usr/bin/python

PROGRAM_VERSION = '0.1.1'

import src.tempy as tempy
from src.Symbol import Symbol
from src.Language import Token,TokenReader,TokenType
from src.Assembler import Assembler
from src.packer import *
from src.globals import *
import argparse
import os.path
import sys
import time

asm = None
parser = None

def lineCallback():
	asm.parse()



def main():
	global asm, parser
	ap = argparse.ArgumentParser(prog=PROGRAM_NAME, usage='%(prog)s [options] module')
	ap.add_argument("module", help="Specify input module to read from srcdir")
	ap.add_argument("-s", "--srcdir", help="Specify source directory", default='./')
	ap.add_argument("-o", "--outdir", help="Specify output directory", default='./')
	ap.add_argument("--sym-only", help="Generate symbol file only", action="store_true")
	ap.add_argument("--create-make-dep", help="Create a dependency file for use with GNU make", action="store_true")
	ap.add_argument("-d","--create-dep", help="Create a dependency file for use with lazymake.py", action="store_true")
	ap.add_argument("-t","--build-time", type=float, default=None)
	
	#Build time is used to determine whether we need to rebuild an imported module's symbol file based on
	#its modification time. If it isn't specified, file modification time of the importing module is used.
	
	#ap.add_argument("-v","--verbose", help="", action="store_true")
	ap.add_argument("--version", help="Print version number", action="store_true")
	
	args = ap.parse_args()
	
	if args.version:
		print PROGRAM_NAME + ' V' + PROGRAM_VERSION
		return
	
	path = args.module.replace('.','/')
	modulePath = os.path.normpath(args.srcdir + '/' + path + '.z').replace('\\','/')
	moduleOutPath = os.path.normpath(args.outdir + '/' + path).replace('\\','/')
	moduleOutDirs = os.path.dirname(moduleOutPath)
	modulePath1 = modulePath;
	
	if not os.path.exists(modulePath):
		modulePath = os.path.normpath(args.srcdir + '/' + path + '/__root__.z')
		moduleOutDirs = moduleOutPath
		if not os.path.exists(modulePath):
			print 'module not found: '+args.module
			return
	try:
		os.makedirs(moduleOutDirs, 0644);
	except:
		pass
	
	sourceFile = moduleOutPath + '.s'
	headerFile = moduleOutPath + '.h'
	symfile = moduleOutPath + '.sym'
	makeDependencyFile = moduleOutPath + '.md'
	dependencyFile = moduleOutPath + '.d'
	
	if args.build_time==None:
		args.build_time = os.path.getmtime(modulePath)
	
	if not os.path.exists(args.outdir):
		os.makedirs(args.outdir) #TODO: make sub-dirs for sub-modules, too... later on
	parser = tempy.Parser(modulePath)
	asm = Assembler(parser, args.module, args.srcdir, args.outdir, args.build_time)

	parser.setLineCallback(lineCallback)
	lines = parser.parse() #1st pass
	#''.join(lines)
	with open(symfile,'wb') as f:
		p = Packer()
		asm.globalScope.toObject(p)
		f.write(p.getData())
		f.close()
	#asm.doImports()
	if not args.sym_only:
		parser.lines = lines
		asm.prepare2ndPass()
		lines = parser.parse() #2nd pass
		with open(sourceFile,'w') as f:
			f.write('.include "'+os.path.basename(headerFile)+'"\n')
			for l in lines:
				f.write(l+'\n')
			f.close()
		with open(headerFile,'wb') as f:
			f.write(asm.getHeader())
			f.close()
			
	if args.create_make_dep:
		with open(makeDependencyFile, 'w') as f:
			f.write(sourceFile + ':\t')
			files = []
			for d in asm.dependencies:
				file = args.srcdir + '/' + d.replace('.','/') + '.z'
				files.append(file)
			f.write(' '.join(files) + '\n\n')
			f.write(' '.join(files) + ':\n')
			f.close()
			
	if args.create_dep:
		with open(dependencyFile, 'w') as f:
			f.write(' '.join(asm.dependencies) + '\n')
			f.close()
	return 0

try:
	sys.exit(main())
except tempy.Error, e:
	if parser == None:
		print 'error: '+e.msg
	else:
		print 'parser error: '+parser.filename+'('+str(parser.lineNumber)+'): ' + e.msg
		print 'near "'+parser.line+'"'
		sys.exit(1)
